﻿using ETSI.ARF.WorldStorage;
using ETSI.ARF.WorldStorage.REST;
using ETSI.ARF.WorldStorage.UI;
using Org.OpenAPITools.Model;
using System;
using System.Collections.Generic;
using UnityEditor;
using UnityEditor.UIElements;
using UnityEngine;
using UnityEngine.UIElements;

namespace Assets.ETSI.ARF.ARF_World_Storage_API.Editor.Windows
{
    public class WorldGraphWindow : EditorWindow
    {

        [HideInInspector] public WorldStorageServer worldStorageServer;
        [HideInInspector] public WorldStorageUser worldStorageUser;

        private ARFGraphView myGraph;

        [MenuItem("ARFWorldStorage/Edit Graph...")]
        public static void ShowWindow()
        {
            GetWindow<WorldGraphWindow>("Graph Editor", true, typeof(SceneView));
        }

        public void OnEnable()
        {
            if (SaveInfo.instance.nodePositions == null)
            {
                SaveInfo.instance.initNodePos(worldStorageServer);
            }

            rootVisualElement.Add(GenerateToolbar());

            ConstructGraphView();
            myGraph.style.top = Length.Percent(11);
            rootVisualElement.Add(myGraph);
        }

        //initiate the graphView Attribute 
        public void ConstructGraphView()
        {
            myGraph = new ARFGraphView
            {
                name = "ARF Graph",
                worldStorageServer = worldStorageServer,
                worldStorageUser = worldStorageUser
            };
            //top offset so that the graph does'nt overlap with the rest of the ui
            myGraph.style.top = Length.Percent(11);
            myGraph.paintWorldStorage();
            myGraph.StretchToParentSize();
            SaveInfo.instance.toReFrame = true;
        }


        void OnGUI()
        {
            EditorGUILayout.BeginVertical();
            EditorGUILayout.Space(24);

            EditorGUI.BeginChangeCheck();
            worldStorageServer = (WorldStorageServer)EditorGUILayout.ObjectField("World Storage Server", worldStorageServer, typeof(WorldStorageServer), false, GUILayout.Width(500));
            if (EditorGUI.EndChangeCheck())
            {
                myGraph.saveElemPositionsInServer();
                rootVisualElement.Remove(myGraph);

                SaveInfo.instance.initNodePos(worldStorageServer);

                ConstructGraphView();
                myGraph.style.top = Length.Percent(11);
                rootVisualElement.Add(myGraph);
            }


            //reframe all elements to see them all
            if (SaveInfo.instance.toReFrame)
            {
                myGraph.FrameAllElements();
                SaveInfo.instance.toReFrame = false;
            }
            EditorGUILayout.EndVertical();
        }
        private bool localAndServerDifferent()
        {
            foreach (ARFNode node in myGraph.nodes)
            {
                float nodeX = node.GetPosition().x;
                float nodeY = node.GetPosition().y;
                if (SaveInfo.instance.nodePositions.ContainsKey(node.GUID)){ 
                    float dataX = SaveInfo.instance.nodePositions[node.GUID].x;
                    float dataY = SaveInfo.instance.nodePositions[node.GUID].y;
                    if ((nodeX != dataX) || (nodeY != dataY))
                    {
                        return true;
                    }
                }
                else
                {
                    return true;
                }
            }
            return false;
        }


        //generate the window's top toolbar
        private Toolbar GenerateToolbar()
        {
            var toolbar = new Toolbar();

            var createNodeT = new Button(clickEvent: () =>
            {
                TrackableWindow.ShowWindow(worldStorageServer, worldStorageUser);
                TrackableWindow.winSingleton.nodePosX = 0;
                TrackableWindow.winSingleton.nodePosY = 0;
            });
            createNodeT.text = "Create Trackable";
            toolbar.Add(createNodeT);

            var createNodeWA = new Button(clickEvent: () =>
            {
                WorldAnchorWindow.ShowWindow(worldStorageServer, worldStorageUser);
                WorldAnchorWindow.winSingleton.nodePosX = 0;
                WorldAnchorWindow.winSingleton.nodePosY = 0;
            });
            createNodeWA.text = "Create World Anchor";
            toolbar.Add(createNodeWA);

            toolbar.Add(new ToolbarSpacer()
            {
                flex = true
            });

            var save = new Button(clickEvent: () =>
            {
                myGraph.saveElemPositionsInServer();
            })
            {
                text = "Save"
            };
            toolbar.Add(save);

            return toolbar;
        }
    }

    public class SaveInfo : ScriptableSingleton<SaveInfo>
    {
        [SerializeField]
        public Dictionary<String, Rect> nodePositions;
        public List<String> linkIds;

        public List<String> elemsToRemove;

        //keep the info of the graph reframe
        public Boolean toReFrame = false;

        public WorldStorageServer worldStorageServer;

        public void initNodePos(WorldStorageServer server)
        {
            worldStorageServer = server;

            instance.nodePositions = new Dictionary<string, Rect>();
            foreach (Trackable track in TrackableRequest.GetAllTrackables(worldStorageServer))
            {
                if (track.KeyvalueTags.ContainsKey("unityAuthoringPosX") && track.KeyvalueTags.ContainsKey("unityAuthoringPosY"))
                {
                    var posX = RoundToNearestHalf(float.Parse(track.KeyvalueTags["unityAuthoringPosX"][0]));
                    var posY = RoundToNearestHalf(float.Parse(track.KeyvalueTags["unityAuthoringPosY"][0]));
                    Rect trackPos = new Rect(posX, posY, 135, 77);
                    instance.nodePositions[track.UUID.ToString()] = trackPos;
                }
                else
                {
                    Rect trackPos = new Rect(0, 0, 135, 77);
                    instance.nodePositions[track.UUID.ToString()] = trackPos;
                }
            }
            foreach (WorldAnchor wa in WorldAnchorRequest.GetAllWorldAnchors(worldStorageServer))
            {
                if (wa.KeyvalueTags.ContainsKey("unityAuthoringPosX") && wa.KeyvalueTags.ContainsKey("unityAuthoringPosY"))
                {
                    var posX = RoundToNearestHalf(float.Parse(wa.KeyvalueTags["unityAuthoringPosX"][0]));
                    var posY = RoundToNearestHalf(float.Parse(wa.KeyvalueTags["unityAuthoringPosY"][0]));
                    Rect waPos = new Rect(posX, posY, 135, 77);
                    instance.nodePositions[wa.UUID.ToString()] = waPos;
                }
                else
                {
                    Rect trackPos = new Rect(0, 0, 135, 77);
                    instance.nodePositions[wa.UUID.ToString()] = trackPos;
                }
            }

            instance.linkIds = new List<string>();
            foreach (WorldLink link in WorldLinkRequest.GetAllWorldLinks(worldStorageServer))
            {
                instance.linkIds.Add(link.UUID.ToString());
            }
            instance.toReFrame = true;

            instance.elemsToRemove = new List<string>();
        }

        //method to predict the position of a node (the float that will be saved in the PositionInfo singleton)
        public static float RoundToNearestHalf(float a)
        {
            return a = Mathf.Round(a * 2f) * 0.5f;
        }


    }
}